"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AddIsSyncedToReadModel1641991678453 = void 0;
const lodash_1 = require("lodash");
class AddIsSyncedToReadModel1641991678453 {
    constructor() {
        this.name = 'AddIsSyncedToReadModel1641991678453';
    }
    async up(queryRunner) {
        await queryRunner.query(`ALTER TABLE "certificate_read_model" ADD "isSynced" boolean`);
        await this.setSyncStatus(queryRunner);
        await queryRunner.query(`ALTER TABLE "certificate_read_model" ALTER COLUMN  "isSynced" SET NOT NULL`);
    }
    async down(queryRunner) {
        await queryRunner.query(`ALTER TABLE "certificate_read_model" DROP COLUMN "isSynced"`);
    }
    async setSyncStatus(queryRunner) {
        const events = await queryRunner.query(`SELECT * FROM certificate_event`);
        const eventsByCertificate = lodash_1.groupBy(events, 'internalCertificateId');
        for (const [internalCertificateId, events] of Object.entries(eventsByCertificate)) {
            const isSynced = this.reduceIsSynced(events);
            await queryRunner.query(`UPDATE "certificate_read_model" SET "isSynced" = $1 WHERE "internalCertificateId" = $2`, [isSynced, internalCertificateId]);
        }
    }
    reduceIsSynced(events) {
        const toPersistCounter = events.reduce((toPersistCounter, event) => {
            switch (event.type) {
                case 'Issued':
                case 'Transferred':
                case 'Claimed':
                    return toPersistCounter + 1;
                case 'ClaimPersisted':
                case 'IssuancePersisted':
                case 'TransferPersisted':
                    return toPersistCounter - 1;
                case 'PersistError':
                    return toPersistCounter;
                default:
                    toPersistCounter;
            }
        }, 0);
        if (toPersistCounter > 0) {
            return false;
        }
        else if (toPersistCounter === 0) {
            return true;
        }
        else {
            throw new Error(`Too many persisted events for certificate ${events[0].internalCertificateId}`);
        }
    }
}
exports.AddIsSyncedToReadModel1641991678453 = AddIsSyncedToReadModel1641991678453;
//# sourceMappingURL=1641991678453-AddIsSyncedToReadModel.js.map