"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificatePersistErrorEvent = exports.CertificateClaimPersistedEvent = exports.CertificateTransferPersistedEvent = exports.CertificateIssuancePersistedEvent = exports.CertificateClaimedEvent = exports.CertificateTransferredEvent = exports.CertificateIssuedEvent = exports.CertificateEventType = exports.isPersistedEvent = exports.isClaimEvent = exports.isIssuePersistedEvent = exports.isIssueEvent = exports.isTransferEvent = void 0;
const version = 1;
const isTransferEvent = (e) => e.type === CertificateEventType.Transferred;
exports.isTransferEvent = isTransferEvent;
const isIssueEvent = (e) => e.type === CertificateEventType.Issued;
exports.isIssueEvent = isIssueEvent;
const isIssuePersistedEvent = (e) => e.type === CertificateEventType.IssuancePersisted;
exports.isIssuePersistedEvent = isIssuePersistedEvent;
const isClaimEvent = (e) => e.type === CertificateEventType.Claimed;
exports.isClaimEvent = isClaimEvent;
const isPersistedEvent = (e) => [
    CertificateEventType.ClaimPersisted,
    CertificateEventType.IssuancePersisted,
    CertificateEventType.TransferPersisted,
    CertificateEventType.PersistError
].includes(e.type);
exports.isPersistedEvent = isPersistedEvent;
var CertificateEventType;
(function (CertificateEventType) {
    CertificateEventType["Issued"] = "Issued";
    CertificateEventType["Transferred"] = "Transferred";
    CertificateEventType["Claimed"] = "Claimed";
    CertificateEventType["IssuancePersisted"] = "IssuancePersisted";
    CertificateEventType["TransferPersisted"] = "TransferPersisted";
    CertificateEventType["ClaimPersisted"] = "ClaimPersisted";
    CertificateEventType["PersistError"] = "PersistError";
})(CertificateEventType = exports.CertificateEventType || (exports.CertificateEventType = {}));
class CertificateIssuedEvent {
    constructor(_id, internalCertificateId, payload) {
        this._id = _id;
        this.internalCertificateId = internalCertificateId;
        this.payload = payload;
        this.type = CertificateEventType.Issued;
        this.version = version;
        this.createdAt = new Date();
    }
    get id() {
        if (this._id === -1) {
            throw new Error('Certificate event has not been saved, and has not set id');
        }
        return this._id;
    }
    static createNew(internalCertificateId, payload) {
        return new CertificateIssuedEvent(-1, internalCertificateId, payload);
    }
}
exports.CertificateIssuedEvent = CertificateIssuedEvent;
class CertificateTransferredEvent {
    constructor(_id, internalCertificateId, payload) {
        this._id = _id;
        this.internalCertificateId = internalCertificateId;
        this.payload = payload;
        this.type = CertificateEventType.Transferred;
        this.version = version;
        this.createdAt = new Date();
    }
    get id() {
        if (this._id === -1) {
            throw new Error('Certificate event has not been saved, and has not set id');
        }
        return this._id;
    }
    static createNew(internalCertificateId, payload) {
        return new CertificateTransferredEvent(-1, internalCertificateId, payload);
    }
}
exports.CertificateTransferredEvent = CertificateTransferredEvent;
class CertificateClaimedEvent {
    constructor(_id, internalCertificateId, payload) {
        this._id = _id;
        this.internalCertificateId = internalCertificateId;
        this.payload = payload;
        this.type = CertificateEventType.Claimed;
        this.version = version;
        this.createdAt = new Date();
    }
    get id() {
        if (this._id === -1) {
            throw new Error('Certificate event has not been saved, and has not set id');
        }
        return this._id;
    }
    static createNew(internalCertificateId, payload) {
        return new CertificateClaimedEvent(-1, internalCertificateId, payload);
    }
}
exports.CertificateClaimedEvent = CertificateClaimedEvent;
class CertificateIssuancePersistedEvent {
    constructor(_id, internalCertificateId, payload) {
        this._id = _id;
        this.internalCertificateId = internalCertificateId;
        this.payload = payload;
        this.type = CertificateEventType.IssuancePersisted;
        this.version = version;
        this.createdAt = new Date();
    }
    get id() {
        if (this._id === -1) {
            throw new Error('Certificate event has not been saved, and has not set id');
        }
        return this._id;
    }
    static createNew(internalCertificateId, payload) {
        return new CertificateIssuancePersistedEvent(-1, internalCertificateId, payload);
    }
}
exports.CertificateIssuancePersistedEvent = CertificateIssuancePersistedEvent;
class CertificateTransferPersistedEvent {
    constructor(_id, internalCertificateId, payload) {
        this._id = _id;
        this.internalCertificateId = internalCertificateId;
        this.payload = payload;
        this.type = CertificateEventType.TransferPersisted;
        this.version = version;
        this.createdAt = new Date();
    }
    get id() {
        if (this._id === -1) {
            throw new Error('Certificate event has not been saved, and has not set id');
        }
        return this._id;
    }
    static createNew(internalCertificateId, payload) {
        return new CertificateTransferPersistedEvent(-1, internalCertificateId, payload);
    }
}
exports.CertificateTransferPersistedEvent = CertificateTransferPersistedEvent;
class CertificateClaimPersistedEvent {
    constructor(_id, internalCertificateId, payload) {
        this._id = _id;
        this.internalCertificateId = internalCertificateId;
        this.payload = payload;
        this.type = CertificateEventType.ClaimPersisted;
        this.version = version;
        this.createdAt = new Date();
    }
    get id() {
        if (this._id === -1) {
            throw new Error('Certificate event has not been saved, and has not set id');
        }
        return this._id;
    }
    static createNew(internalCertificateId, payload) {
        return new CertificateClaimPersistedEvent(-1, internalCertificateId, payload);
    }
}
exports.CertificateClaimPersistedEvent = CertificateClaimPersistedEvent;
class CertificatePersistErrorEvent {
    constructor(_id, internalCertificateId, payload) {
        this._id = _id;
        this.internalCertificateId = internalCertificateId;
        this.payload = payload;
        this.type = CertificateEventType.PersistError;
        this.version = version;
        this.createdAt = new Date();
    }
    get id() {
        if (this._id === -1) {
            throw new Error('Certificate event has not been saved, and has not set id');
        }
        return this._id;
    }
    static createNew(internalCertificateId, payload) {
        return new CertificatePersistErrorEvent(-1, internalCertificateId, payload);
    }
}
exports.CertificatePersistErrorEvent = CertificatePersistErrorEvent;
//# sourceMappingURL=Certificate.events.js.map