"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var BlockchainSynchronizeTask_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockchainSynchronizeTask = void 0;
const common_1 = require("@nestjs/common");
const bull_1 = require("@nestjs/bull");
const repository_keys_1 = require("../repositories/repository.keys");
const synchronize_strategy_1 = require("./strategies/synchronize.strategy");
const SYNCHRONIZATION_CONCURRENCY = 1;
let BlockchainSynchronizeTask = BlockchainSynchronizeTask_1 = class BlockchainSynchronizeTask {
    constructor(certEventRepo, synchronizeStrategy) {
        this.certEventRepo = certEventRepo;
        this.synchronizeStrategy = synchronizeStrategy;
        this.logger = new common_1.Logger(BlockchainSynchronizeTask_1.name);
    }
    async synchronizeWithBlockchain() {
        try {
            this.logger.debug(`Synchronization with blockchain started at ${new Date()}`);
            let i = 0;
            while (i < 50) {
                const events = await this.certEventRepo.findAllToProcess({
                    limit: 500
                });
                if (events.length === 0) {
                    break;
                }
                await this.synchronizeStrategy.synchronize(events);
                i += 1;
            }
            this.logger.debug(`Synchronization with blockchain finished at ${new Date()}`);
        }
        catch (e) {
            this.logger.error(`Error occurred while synchronizing certificates with blockchain: ${e.message}`, e.stack);
            throw e;
        }
    }
};
__decorate([
    bull_1.Process({ concurrency: SYNCHRONIZATION_CONCURRENCY }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], BlockchainSynchronizeTask.prototype, "synchronizeWithBlockchain", null);
BlockchainSynchronizeTask = BlockchainSynchronizeTask_1 = __decorate([
    common_1.Injectable(),
    bull_1.Processor(repository_keys_1.SYNCHRONIZE_QUEUE_NAME),
    __param(0, common_1.Inject(repository_keys_1.CERTIFICATE_EVENT_REPOSITORY)),
    __param(1, common_1.Inject(synchronize_strategy_1.SYNCHRONIZE_STRATEGY)),
    __metadata("design:paramtypes", [Object, Object])
], BlockchainSynchronizeTask);
exports.BlockchainSynchronizeTask = BlockchainSynchronizeTask;
//# sourceMappingURL=blockchain-synchronize.task.js.map