"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var OnChainWatcher_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockchainEventType = exports.OnChainWatcher = void 0;
const common_1 = require("@nestjs/common");
const issuer_1 = require("@energyweb/issuer");
const config_1 = require("@nestjs/config");
const blockchain_properties_service_1 = require("../blockchain-properties.service");
const transaction_poll_service_1 = require("../certificate-operations/transaction-poll.service");
let OnChainWatcher = OnChainWatcher_1 = class OnChainWatcher {
    constructor(configService, blockchainPropertiesService, transactionPollService) {
        this.configService = configService;
        this.blockchainPropertiesService = blockchainPropertiesService;
        this.transactionPollService = transactionPollService;
        this.logger = new common_1.Logger(OnChainWatcher_1.name);
    }
    async onModuleInit() {
        this.logger.debug('Setting up on-chain certificate watcher');
        const { web3, registry } = await this.blockchainPropertiesService.getWrapped();
        this.provider = web3;
        this.registry = registry;
        this.provider.pollingInterval =
            Number(this.configService.get('BLOCKCHAIN_POLLING_INTERVAL')) ||
                this.provider.pollingInterval;
        this.provider.on(this.registry.filters.IssuanceSingle(null, null, null), (event) => this.processEvent(BlockchainEventType.IssuanceSingle, event));
        this.provider.on(this.registry.filters.IssuanceBatch(null, null, null), (event) => this.processEvent(BlockchainEventType.IssuanceBatch, event));
        this.provider.on(this.registry.filters.TransferSingle(null, null, null, null, null), (event) => this.processEvent(BlockchainEventType.TransferSingle, event));
        this.provider.on(this.registry.filters.TransferBatch(null, null, null, null, null), (event) => this.processEvent(BlockchainEventType.TransferBatch, event));
        this.provider.on(this.registry.filters.TransferBatchMultiple(null, null, null, null, null), (event) => this.processEvent(BlockchainEventType.TransferBatchMultiple, event));
        this.provider.on(this.registry.filters.ClaimSingle(null, null, null, null, null, null), (event) => this.processEvent(BlockchainEventType.ClaimSingle, event));
        this.provider.on(this.registry.filters.ClaimBatch(null, null, null, null, null, null), (event) => this.processEvent(BlockchainEventType.ClaimBatch, event));
        this.provider.on(this.registry.filters.ClaimBatchMultiple(null, null, null, null, null, null), (event) => this.processEvent(BlockchainEventType.ClaimBatchMultiple, event));
    }
    async processEvent(eventType, rawEvent) {
        this.logger.debug(`Processing event ${eventType}: ${JSON.stringify(rawEvent)}`);
        const event = await issuer_1.CertificateUtils.decodeEvent(eventType, rawEvent, this.registry);
        switch (eventType) {
            case BlockchainEventType.IssuanceSingle:
                this.logEvent(BlockchainEventType.IssuanceSingle, [event._id.toNumber()]);
                this.transactionPollService.saveNewCertificate(event.transactionHash, [
                    event._id.toNumber()
                ]);
                break;
            case BlockchainEventType.IssuanceBatch:
                const ids = event._ids.map((id) => id.toNumber());
                this.logEvent(BlockchainEventType.IssuanceBatch, ids);
                this.transactionPollService.saveNewCertificate(event.transactionHash, ids);
                break;
            case BlockchainEventType.TransferSingle:
                this.logEvent(BlockchainEventType.TransferSingle, [event.id.toNumber()]);
                break;
            case BlockchainEventType.TransferBatch:
            case BlockchainEventType.TransferBatchMultiple:
                event.ids.forEach((id) => {
                    this.logEvent(BlockchainEventType.TransferBatch, [id.toNumber()]);
                });
                break;
            case BlockchainEventType.ClaimSingle:
                this.logEvent(BlockchainEventType.ClaimSingle, [event._id.toNumber()]);
                break;
            case BlockchainEventType.ClaimBatch:
            case BlockchainEventType.ClaimBatchMultiple:
                event._ids.forEach((id) => {
                    this.logEvent(BlockchainEventType.ClaimBatch, [id.toNumber()]);
                });
                break;
            default:
                this.logger.log(`No handlers found for event: ${eventType} on Certificate ${event._id.toNumber()}`);
        }
        await this.transactionPollService.saveTransactionAsProcessed(event.transactionHash);
    }
    logEvent(type, ids) {
        this.logger.log(`Detected a new event: ${type} on Certificate ${JSON.stringify(ids)}`);
    }
    async onApplicationShutdown() {
        this.logger.debug('Shutting down on-chain certificate watcher');
        this.provider.off(this.registry.filters.IssuanceSingle(null, null, null));
        this.provider.off(this.registry.filters.IssuanceBatch(null, null, null));
        this.provider.off(this.registry.filters.TransferSingle(null, null, null, null, null));
        this.provider.off(this.registry.filters.TransferBatch(null, null, null, null, null));
        this.provider.off(this.registry.filters.ClaimSingle(null, null, null, null, null, null));
        this.provider.off(this.registry.filters.ClaimBatch(null, null, null, null, null, null));
    }
};
OnChainWatcher = OnChainWatcher_1 = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [config_1.ConfigService,
        blockchain_properties_service_1.BlockchainPropertiesService,
        transaction_poll_service_1.TransactionPollService])
], OnChainWatcher);
exports.OnChainWatcher = OnChainWatcher;
var BlockchainEventType;
(function (BlockchainEventType) {
    BlockchainEventType["IssuanceSingle"] = "IssuanceSingle";
    BlockchainEventType["IssuanceBatch"] = "IssuanceBatch";
    BlockchainEventType["TransferSingle"] = "TransferSingle";
    BlockchainEventType["TransferBatch"] = "TransferBatch";
    BlockchainEventType["ClaimSingle"] = "ClaimSingle";
    BlockchainEventType["ClaimBatch"] = "ClaimBatch";
    BlockchainEventType["TransferBatchMultiple"] = "TransferBatchMultiple";
    BlockchainEventType["ClaimBatchMultiple"] = "ClaimBatchMultiple";
})(BlockchainEventType = exports.BlockchainEventType || (exports.BlockchainEventType = {}));
//# sourceMappingURL=on-chain-certificates.listener.js.map